import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: root
    width: 400
    height: 250

    ColumnLayout {
        id: mainLayout
        anchors.fill: parent
        anchors.margins: 11
        spacing: 16

        GroupBox {
            id: rangeBox
            title: "Intensity range"
            Layout.fillWidth: true

            RowLayout {
                id: rangeRowLayout
                anchors.fill: parent

                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "From:"
                }
                SpinBox {
                    id: rangeFrom
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 0
                    editable: true
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    text: "To:"
                }
                SpinBox {
                    id: rangeTo
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 255
                    editable: true
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
            }
        }

        GroupBox {
            id: voxelsBox
            title: "Voxels in intensity range"
            Layout.fillWidth: true

            RowLayout {
                id: voxelsRowLayout
                anchors.fill: parent

                Label {
                    text: "No. of voxels:"
                }
                TextField {
                    id: nvoxelsText
                    placeholderText: "# voxels"
                    Layout.fillWidth: true
                }
                Button {
                    text: "Count"
                    onClicked: nvoxelsText.text = root.onCountVoxels(rangeFrom.value, rangeTo.value)
                }
            }
        }

        Item {  // spacer item
            Layout.fillHeight: true
        }

        RowLayout {
            id: buttonsRow

            Button {
                id: undoBtn
                text: "Undo"
                onClicked: root.onUndo()
            }
            Item { Layout.fillWidth: true }
            Button {
                id: quitBtn
                text: "Quit"
                onClicked: BrainVoyager.FinishPlugin()
            }
            Button {
                id: invertBtn
                text: "Invert"
                onClicked: root.onInvert()
            }
        }
    }        // end of ColumnLayout

    Component.onCompleted: {
        console.log("Plugin ready") // + "  parent width x height: " + parent.width + " " + parent.height)
        BrainVoyager.PrintToLog("Plugin GUI> Hello from QML GUI Plugin")
    }
    
    function onCountVoxels(irange_from, irange_to)
    {
        BrainVoyager.PrintToLog("Plugin GUI> Dialog's Count Voxels button clicked");

        BrainVoyager.SetPluginStringParameter("Command", "CountVoxels");
        BrainVoyager.SetPluginIntParameter("MinimumIntensity", irange_from);
        BrainVoyager.SetPluginIntParameter("MaximumIntensity", irange_to);

        BrainVoyager.ExecutePlugin();

        var nrOfVoxels = BrainVoyager.GetPluginIntParameter("NumberOfCountedVoxels"); // get result from plugin

        return nrOfVoxels;
    }

    function onInvert()
    {
        BrainVoyager.PrintToLog("Plugin GUI> Dialog's Invert button clicked");

        BrainVoyager.SetPluginStringParameter("Command", "Invert");
        BrainVoyager.SetPluginIntParameter("MinimumIntensity", rangeFrom.value);
        BrainVoyager.SetPluginIntParameter("MaximumIntensity", rangeTo.value);

        BrainVoyager.ExecutePlugin();
    }
    function onUndo()
    {
        BrainVoyager.PrintToLog("Plugin GUI> Dialog's Undo button clicked - restoring original VMR intensities");

        BrainVoyager.SetPluginStringParameter("Command", "Undo");

        BrainVoyager.ExecutePlugin();
    }
}
