import numpy as np
import bva

def get_doc_and_mesh():
    doc = brainvoyager.adoc                   # A VMR-with-Mesh must be the current document
    if doc is None:
        return (None, None)
    mesh = doc.current_mesh
    if mesh is None:
        return (doc, None)
    else:
        return (doc, mesh)

# this fn is an example to show that vertex coordinates can be directly changed in place
# via the returned numpy arrays "wrapping" original BV C pointers. The modified mesh should
# usually not be saved, however, since changing vertex coordinates breaks the match with VMR
def move_mesh(dx, dy, dz):
    vertices_x, vertices_y, vertices_z = bva.mesharrays(mesh) # BV internal axis convention
    # vertices_y, vertices_z, vertices_x = bv.mesharrays(mesh) # use to map to Dicom/TAL axis convention
    print('vertices x-coord - shape: ', vertices_x.shape)
    print('vertices x-coord - dtype: ', vertices_x.dtype.name)
    print('vertices x-coord - values:', vertices_x)
    vertices_x += dx
    vertices_y += dy
    vertices_z += dz
    #ok = bv.set_mesharrays(mesh, vertices_x, vertices_y, vertices_z) # if new arrays created, use this fn
    mesh.recreate_geometry()

doc, mesh = get_doc_and_mesh()
if not mesh is None:
    move_mesh(-10.0, 10.0, 0.0)
    mesh.update_appearance()
