import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt

doc = brainvoyager.active_document              # A FMR must be the current document 
dimX = doc.dim_x
dimY = doc.dim_y
dimZ = doc.dim_z
dimT = doc.n_volumes

fmr_data = doc.get_fmr_data_as_float_list()    # returns list of doubles (for doubles: 'get_fmr_data_as_float64_list')
data   = np.array(fmr_data, dtype='float32')   # convert Python list to numpy array
data4D = data.reshape(dimT, dimZ, dimY, dimX)  # reshape 1D to 4D array

def calculate_tSNR(array4D):
    m    = array4D.mean(axis=0)
    sd   = array4D.std(axis=0)
    tsnr = m / sd
    return m, sd, tsnr

# plot a slice with original, mean, sdev and TSNR image
def plotslice(sl):
    plt.subplot(221)
    plt.imshow(data4D[0][sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.title('Original image (first volume)')
    plt.subplot(222)
    plt.imshow(M[sl])
    plt.colorbar()
    plt.title('Mean time course image')
    plt.subplot(223)
    plt.imshow(SD[sl])
    plt.colorbar()
    plt.title('StdDev time course image')
    plt.subplot(224)
    plt.imshow(TSNR[sl])
    plt.colorbar()
    plt.title('TSNR image')
    plt.show()

M, SD, TSNR = calculate_tSNR(data4D)
plotslice(int(dimZ/2))
