import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt
import bva

doc = brainvoyager.active_document                   
if doc.is_fmr() == False:                              # current document must be FMR 
    raise ValueError('Active document is not an FMR')
dimX = doc.dim_x
dimY = doc.dim_y
dimZ = doc.dim_z
dimT = doc.n_volumes

tp = 10 # process volume at time point tp (for full data, iterate over dimT or use full tc access)
fmr_vol = bva.fmrarray_t(doc, tp) # read first functional volume

data3D   = fmr_vol.reshape(dimZ, dimY, dimX)          # reshape 1D to 3D array
data3D2  = np.copy(data3D)
data3Dsm = ndimage.gaussian_filter(data3D2, sigma=2)  # smooth volume - to preserve original, use "data3D.copy()"

# save smoothed volume back to original STC volume in BV's memory -> this updates also "fmr_vol" and "data3D"!
ok = bva.set_fmrarray_t(doc, tp, data3Dsm)

def savefmr(doc):
    fmrfilename = doc.path_file_name
    new_fmrfilename = fmrfilename.rsplit('.', 1)[0] # remove file extension
    new_fmrfilename += '_pytest.fmr'
    doc.save_fmr_stc_from_mem(new_fmrfilename);
    return new_fmrfilename;

def plotslice(sl):  # plot original and result for a given slice
    plt.subplot(221)
    plt.imshow(data3D2[sl])
    plt.colorbar()
    plt.subplot(222)
    plt.imshow(data3Dsm[sl])
    plt.colorbar()
    plt.subplot(223)
    plt.imshow(data3D2[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.subplot(224)
    plt.imshow(data3Dsm[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.show()

# if this would be a real (pre-)processing, one would save result to disk
newfmrfilename = savefmr(doc)
newdoc = brainvoyager.OpenDocument(newfmrfilename)  # open saved FMR

plotslice(13)
