import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt

doc = brainvoyager.active_document
if doc.is_vmr() == False:                         # current document should be a VMR
    raise ValueError('current doc is not a VMR')
n_voxels = doc.dim_x * doc.dim_y * doc.dim_z

# "get_volume_data" functions operate much faster than voxelwise scripting access (see VolumeVMR_SlowAccess.py)
# even more efficient is a direct conversion/access to numpy ndarray - see 'vmr_fast_array_access.py"
#
volume_data = doc.get_volume_data_as_float_list() # returns flat list of voxel intensities
#volume_data = doc.get_volume_data_as_int_list()

arr = np.array(volume_data)                         # convert list to numpy array
arr2 = arr.reshape(doc.dim_x, doc.dim_y, doc.dim_z) # reshape 1D to 3D array

plt.subplot(221)
plt.imshow(arr2[doc.dim_x//2])
plt.subplot(222)
plt.imshow(arr2[doc.dim_x//2], cmap=plt.cm.gray)
arr3 = ndimage.gaussian_filter(arr2, sigma=3)    # smooth volume with gaussian kernel
plt.subplot(223)
plt.imshow(arr3[doc.dim_x//2])
plt.subplot(224)
plt.imshow(arr3[doc.dim_x//2], cmap=plt.cm.gray)
plt.show()
