import numpy as np
import scipy.ndimage as ndimage
import bva  # supports computationally powerful plugin development by providing numpy array interface to internal BV data

doc_vmr = brainvoyager.adoc               # A VMR should be the current document 
n_voxels = doc_vmr.dim_x * doc_vmr.dim_y * doc_vmr.dim_z

# "vmrarray" function of 'bva' module provides direct access to VMR volume as ndarray
# this solves the very slow voxelwise access via scripting (see PythonScripts/VolumeVMR_SlowAccess.py)
#
arr = bva.vmrarray(doc_vmr)

arrf = arr.astype(np.float32)       # convert to float32 array
arr2 = arrf.reshape(doc_vmr.dim_x, doc_vmr.dim_y, doc_vmr.dim_z) # reshape 1D to 3D array
arr3 = ndimage.gaussian_filter(arr2, sigma=3)     # smooth volume
arr3_ui8 = arr3.astype(np.uint8)                  # convert smoothed arrat back to uint8 array
bva.set_vmrarray(doc_vmr, arr3_ui8)               # copy array to BV internal volume (C array)
doc_vmr.update_view(True)                         # update display to see inverted intensities in BV doc view
