import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt
import bva


doc_vmr = brainvoyager.adoc                       # A VMR should be the current document 
dimx = doc_vmr.dim_x
dimy = doc_vmr.dim_y
dimz = doc_vmr.dim_z
n_voxels = dimx * dimy * dimz

# "vmrarray" function of 'bva' module provides direct access to VMR volume as ndarray
# this solves the very slow voxelwise access via scripting (see PythonScripts/VolumeVMR_SlowAccess.py)
#
arr = bva.vmrarray(doc_vmr)

arrf = arr.astype(np.float32)                     # convert to float32 array
arr2 = arrf.reshape(dimx, dimy, dimz)             # reshape 1D to 3D array
arr3 = ndimage.gaussian_filter(arr2, sigma=4)    # smooth volume
arr4 = arr2 - arr3                                # high-pass = img - low-pass
amin = arr4.min()
amax = arr4.max()
print(f'high-pass: min: {amin}, max: {amax}')
arr4 = (arr4 - amin) / (amax - amin) * 225.0
arr3_ui8 = arr3.astype(np.uint8)                  # convert smoothed array back to uint8 array
arr4_ui8 = arr4.astype(np.uint8)
bva.set_vmrarray(doc_vmr, arr4_ui8)               # copy array to BV internal volume (C array)
doc_vmr.update_view(True)                         # update display to see inverted intensities in BV doc view

def plotslice(sl):  # plot result for a given slice
    plt.subplot(321)
    plt.imshow(arr2[sl])
    plt.colorbar()
    plt.subplot(322)
    plt.imshow(arr2[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.subplot(323)
    plt.imshow(arr3[sl])
    plt.colorbar()
    plt.subplot(324)
    plt.imshow(arr3[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.subplot(325)
    plt.imshow(arr4[sl])
    plt.colorbar()
    plt.subplot(326)
    plt.imshow(arr4[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.show()

plotslice(140)
