#
# This script shows how a mesh can be smoothed and inflated.
# It is assumed that a "*_RECO.srf" mesh has been loaded
#
#  author: Rainer Goebel
#

def morph_mesh():
    doc_vmr = brainvoyager.active_document
    if doc_vmr is None:
        return
    mesh = doc_vmr.current_mesh
    if mesh is None:
        return
    mesh.morphing_update_interval = 25
    # mesh.smooth_geometry_basic(30, 0.07);   # standard smoothing - mesh will shrink
    mesh.smooth_geometry(50, 0.07)  # special "high-frequency" smoothing removing jags of reconstructed voxel borders without shrinking mesh
    mesh.calculate_curvature()
    mesh.smooth_current_map(5)
    mesh.mesh_scene.update_viewer()
    new_name = remove_extension(mesh.file_name) + 'SM.srf'
    print(new_name)
    mesh.save_as(new_name)
    mesh.inflate_geometry(500, 0.8, ''); # if '' used for 3rd param (area reference mesh), the current mesh at the moment the fn is called is used to calculate ref mesh area

# helpers
def remove_extension(filename):
    ext_length = get_extension_length(filename)
    name = ''
    for i in range(len(filename) - ext_length):
        letter = filename[i]
        name += letter
    return name

def get_extension_length(filename):
    i = len(filename) - 1
    while i > 0:
        letter = filename[i]
        if letter == ".":
            dot_pos = i
            break
        i -= 1
    extension_length = len(filename) - dot_pos
    return extension_length

morph_mesh()
