
# Example of basic preprocessing steps for a single VMR document
# More realistic preprocessing would loop over subjects and sessions, or would use the Data Analysis Manager.
# For information about the parameters of the commands, see topic "VMR Processing Commands"
# in the "Scripting" chapter of the "Python Developer Guide"

# we simply use current doc, which should be a non-preprocessed VMR document
# usually the document is created or opened via script command

doc_vmr = brainvoyager.active_document

# The VMR data can be iso-voxeled, (only executes if needed)
path_file = doc_vmr.path_file_name
new_path_file = path_file[0:-4] + "_ISO.vmr"
iso_performed = doc_vmr.transform_to_std_isovoxel(2, new_path_file)
if iso_performed:
    doc_vmr = bv.open(new_path_file, True) # 'True' -> close current doc
    print(new_path_file)
else:
    new_path_file = path_file # nothing done, keep original file name

# The VMR data can be oriented into BV's standard sagittal space (if needed)
path_file = doc_vmr.path_file_name
new_path_file = path_file[0:-4] + "_SAG.vmr"
sag_performed = doc_vmr.transform_to_std_sag(new_path_file)
if sag_performed:
    doc_vmr = bv.open(new_path_file, True) # 'True' -> close current doc
    print(new_path_file)
else:
    new_path_file = path_file # nothing done, keep original file name

# correct intensity inhomogeneities including brain extraction
doc_vmr.correct_intensity_inhomogeneities()

MNI_space = True
TAL_space = False

if MNI_space:
    # transform VMR from native space to MNI152 space
    doc_vmr.normalize_to_mni_space()

if TAL_space:
    # automatically convert VMR from native space to ACPC and Talairach space
    doc_vmr.auto_acpc_tal_transformation()

# documents can be identified by name and selectively closed if desired
n_docs = brainvoyager.documents.size()
for idx in range(0, n_docs):
    doc = brainvoyager.documents.item(idx)
    print(doc.file_name)

# TAL normalization: close the original (inhomogeneity corrected) VMR
if TAL_space:
    doc_vmr.close()

# get MNI or TAL document object to further work in normalized space
doc_normalized = brainvoyager.active_document
print(doc_normalized.path_file_name)
