import numpy as np
#import matplotlib as mpl
#mpl.use('macosx')    # use() must be called before pyplot can be imported
#print(mpl.get_backend())
import matplotlib.pyplot as plt

doc = brainvoyager.active_document   # A VMR should be the current document 

if doc == None:
    raise ValueError('No active document!')  # terminates running script
if doc.is_vmr() == False: 
    raise ValueError('active document is not a VMR!')

print(f'VMR dims - dim_x={doc.dim_x} dim_y={doc.dim_y} dim_z={doc.dim_z}')
    
n_voxels = doc.dim_x * doc.dim_y * doc.dim_z

# Run this only if you have a couple of minutes... inverts the intensities of VMR
# The use of voxel-wise "GetVoxelIntensity" BV funtion in large nested loops is VERY slow!
# For plugin development, fast access to arrays is directly provided (see e.g. VMR_FastArrayAccess.py)

volume = np.empty(n_voxels, dtype=np.float32)
vol3D  = volume.reshape(doc.dim_x, doc.dim_y, doc.dim_z)

for x in range(doc.dim_x):
    #if bv.escape():
    #    break
    print(f'get voxel inensities, outer loop {x+1}')  # slows processing but useful to see (slow) access 
    for y in range(doc.dim_y):
        for z in range(doc.dim_z):
            vol3D[x, y, z] = doc.get_voxel_intensity(x, y, z)

vol3D = 225.0 - vol3D  # vectorized math is fast!

for x in range(doc.dim_x):
    #if bv.escape():
    #    break
    print(f'set voxel intensities, outer loop {x+1}')  # slows processing but useful to see (slow) access 
    for y in range(doc.dim_y):
        for z in range(doc.dim_z):
            doc.set_voxel_intensity(x, y, z, round(vol3D[x, y, z]))

doc.update_view()            # update doc view to show result

# show slices using matplotlib
vol3D = 225.0 - vol3D             # undo intensity flip
vol3Dsag = np.swapaxes(vol3D,1,2) # to get upright SAG view
plt.subplot(223)
plt.imshow(vol3Dsag[doc.dim_x//2], cmap=plt.cm.gray)
plt.subplot(221)
plt.imshow(vol3Dsag[doc.dim_x//2])
vol3Dcor = np.swapaxes(vol3D,0,2) # to get upright COR view
plt.subplot(222)
plt.imshow(vol3Dcor[:,doc.dim_y//2,:])
vol3Dtra = np.swapaxes(vol3D,0,1) # to get upright TRA view
plt.subplot(224)
plt.imshow(vol3Dtra[:,:,doc.dim_z//2])
plt.show()
