import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: root
    width: 700
    height: 500
    padding: 20
    property color extra_bg_col
    property color extra_hl_col

    ColumnLayout {
        anchors.fill: parent
        spacing: 16
        
        RowLayout {

            GroupBox {
                Layout.fillWidth: true
                Layout.fillHeight: true
                Layout.minimumHeight: 140
                Layout.maximumHeight: 140
                title: "Method"
                
                ColumnLayout {
                    
                    RadioButton {
                        id: methodGeneralizedOp
                        checked: true
                        text: "Generalized PPI (McLaren et. al., 2012)"
                        onClicked: root.selectGenPPI()
                    }
                    RadioButton {
                        id: methodStandardOp
                        text: "Standard PPI (Friston et. al., 1997)"
                        onClicked: root.selectStandardPPI()
                    }                
                }
            }
            
            GroupBox {
                Layout.fillWidth: true      
                Layout.fillHeight: true   
                Layout.minimumHeight: 140
                Layout.maximumHeight: 140       
                title: "Options"
                
                RowLayout {
                    
                    ColumnLayout {
                        CheckBox {
                            id: addMCPredsOp
                            text: "Add MC predictors"
                        }
                        CheckBox {
                            id: saveAsMDMOp
                            checked: true
                            text: "Save as MDM"
                        }
                    }
                    ColumnLayout {
                        CheckBox {
                            id: zTransformMCPredsOp
                            text: "z-transform MC predictors"
                        }                       
                        CheckBox {
                            id: specifyPredNameOp
                            text: "Specify predictor name"
                        }
                        CheckBox {
                            id: prtContainsBaselineOp
                            text: "Protocol contains baseline"
                            checked: true
                        }
                    }
                }
            }
        }
        
        // VTCs and VOIs list views - label and groupbox with list view in column layout (we do not use label / title from group box due to bad look on macOS (OS theme)
        RowLayout {
            Layout.fillHeight: true
            spacing: 16

            // VTCs list view
            ColumnLayout {
                Layout.fillHeight: true
                Layout.minimumHeight: 120

                Label {
                    text: "List of VTCs:"
                }

                GroupBox {
                    id: vtcsGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: vtcsListView
                        anchors.fill: parent
                        spacing: 6
                        model: vtcsModel
                        delegate: Label {
                            text: idx + ": " + name
                            MouseArea {
                                id: vtcsMouseArea
                                anchors.fill: parent
                                onClicked: {
                                    console.log("Clicked item: " + (index+1))
                                    vtcsListView.currentIndex = index
                                }
                            }
                        }
                        highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) vtcsGrpBx.background.color = root.extra_bg_col
                }
            }
            
            // VOIs list view
            ColumnLayout {
                Layout.fillHeight: true

                Label {
                    text: "List of VOIs:"
                }

                GroupBox {
                    id: voisGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: voisListView
                        anchors.fill: parent
                        spacing: 6
                        model: voisModel
                        delegate: Label {
                            text: idx + ": " + name
                            MouseArea {
                                id: voisMouseArea
                                anchors.fill: parent
                                onClicked: {
                                    console.log("Clicked item: " + (index+1))
                                    voisListView.currentIndex = index
                                }
                            }
                        }
                        highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) voisGrpBx.background.color = root.extra_bg_col
                }
            }
        }
        
        // 3 list views for weights
        RowLayout {
            Layout.fillHeight: true
            spacing: 16

            // Weight -1 list view
            ColumnLayout {
                Layout.fillHeight: true
                Layout.minimumHeight: 120

                RowLayout { // to center Label
                    Item { Layout.fillWidth: true }
                    Label { id: weightMinLbl; text: "Weight -1"; enabled: false }
                    Item { Layout.fillWidth: true }
                }

                GroupBox {
                    id: weightMinusOneGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: weightMinusOneListView
                        anchors.fill: parent
                        spacing: 6
                        model: weightMinusOneModel
                        delegate: Label {
                            text: idx + ": " + name
                        }
                        //highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) weightMinusOneGrpBx.background.color = root.extra_bg_col
                }
            }
            
            // Weight 0 list view
            ColumnLayout {
                Layout.fillHeight: true

                RowLayout { // to center Label
                    Item { Layout.fillWidth: true }
                    Label { id: weightZeroLbl; text: "Weight 0"; enabled: false }
                    Item { Layout.fillWidth: true }
                }

                GroupBox {
                    id: weightZeroGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: weightZeroListView
                        anchors.fill: parent
                        spacing: 6
                        model: weightZeroModel
                        delegate: Label {
                            text: idx + ": " + name
                        }
                        //highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) weightZeroGrpBx.background.color = root.extra_bg_col
                }
            }
            
            // Weight 1 list view
            ColumnLayout {
                Layout.fillHeight: true

                RowLayout { // to center Label
                    Item { Layout.fillWidth: true }
                    Label { id: weightPlusLbl; text: "Weight 1"; enabled: false }
                    Item { Layout.fillWidth: true }
                }

                GroupBox {
                    id: weightOneGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: weightOneListView
                        anchors.fill: parent
                        spacing: 6
                        model: weightOneModel
                        delegate: Label {
                            text: idx + ": " + name
                        }
                        //highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) weightOneGrpBx.background.color = root.extra_bg_col
                }
            }
        }

        RowLayout {
            id: buttonsRow
            spacing: 8
            
            Item { Layout.fillWidth: true }
            
            Button {
                id: addSubjectBtn
                text: "Add Subject"
                onClicked: root.onAddSubject()
            }
            Button {
                id: loadBtn
                text: "Load..."
                onClicked: root.onLoad()
            }
            Button {
                id: saveAsBtn
                text: "Save As..."
                onClicked: root.onSaveAs()
            }
            Button {
                id: clearBtn
                text: "Clear"
                onClicked: root.onClear()
            }
            Button {
                id: quitBtn
                text: "Quit"
                onClicked: bv.FinishPlugin() // required for plugin GUI frontend, use CloseDialog() for pure GUI scripts 
            }
            Button {
                id: invertBtn
                text: "GO"
                onClicked: root.onGo()
            }
        }
    }
    
    // Here we define some data models (essentially lists) that are filled with dummy values (entries can be removed)
    ListModel {
        id: voisModel

        ListElement {
            idx: 1
            name: "VOI 1 - very very very long"
        }
        ListElement {
            idx: 2
            name: "VOI 2"
        }
    }
    ListModel {
        id: vtcsModel
        ListElement {
            idx: 1
            name: "VTC 1"
            path: "/dummy/path"
        }
    }
    ListModel {
        id: weightMinusOneModel
    }
    ListModel {
        id: weightZeroModel
    }
    ListModel {
        id: weightOneModel
    }
        
    Component.onCompleted: {
        bv.SetGUIScriptWindowTitle("PPI Plugin") // set title of GUI Script window - if GUI is used as plugin frontend, title will be set from plugin code
        
        console.log("Plugin ready") // + "  parent width x height: " + parent.width + " " + parent.height)
        bv.PrintToLog("Plugin GUI> PPI GUI plugin running...")
        // for final version: clear models in case they contain dummy data
        //voisModel.clear()
        //vtcsModel.clear()
        
        // define colors for list view background and highlight
        if(bv.BVTheme) {
            if(bv.DarkMode) {
                root.extra_bg_col = Qt.rgba(0.08, 0.08, 0.08, 1.0)
                root.extra_hl_col = Qt.rgba(0.15, 0.35, 0.6, 1.0)
            }
            else {
                root.extra_bg_col = Qt.rgba(0.86, 0.86, 0.86, 1.0)
                root.extra_hl_col = Qt.rgba(0.3, 0.7, 1.0,  1.0)
            }
        }
        else { // OS theme
            if(bv.DarkMode) root.extra_hl_col = Qt.rgba(0.15, 0.35, 0.6, 1.0)
            else            root.extra_hl_col = Qt.rgba(0.3, 0.6, 1.0,  1.0)
        }
    }
    
    function onSaveAs()
    {
        bv.PrintToLog("Script> 'Save As..' button clicked");
        bv.SetPluginStringParameter("Command", "Save");
        bv.ExecutePlugin();
    }
    
    function onLoad()
    {
        bv.PrintToLog("Script> 'Load' button clicked");
        bv.SetPluginStringParameter("Command", "Load");
        weightMinusOneModel.clear(); weightZeroModel.clear(); weightOneModel.clear();
        vtcsModel.clear(); voisModel.clear();
        bv.SetPluginStringParameter("VTCFileList", $find("tbrVTCFileList").plainText);
        bv.SetPluginStringParameter("VOIFileList", $find("tbrVOIFileList").plainText);
        bv.ExecutePlugin();
    
        var CondsLW_zero1 = $find("listWidget_zero"); // changed
    
        // get result from plugin
        var newVTCFileList = $getstring("VTCFileList");
        var newVOIFileList = $getstring("VOIFileList");
        var prt_cond_header = $getstring("prt_cond_header");
        var saveWeights = $getint("saveWeights");
    
        // update lists
        $find("tbrVTCFileList").plainText = newVTCFileList;
        $find("tbrVOIFileList").plainText = newVOIFileList;
    
        //if(($find("rb_StandardPPI").checked) & (saveWeights == 0)){
        if($find("rb_StandardPPI").checked){
            var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
            for(var c=0; c<NrOfConditions; c++)
            {
                var CondNameID = "CondName-" + (c+1);
                var CondName = $getstring(CondNameID);
                CondsLW_zero1.addItem(CondName);
            }
        }
    }

    function readWeights()
    {
        bv.PrintToLog("Script> Reading weights");
    
        // count number of items in list
        var NCOND = new Array();
        NCOND[0] = weightMinusOneModel.count;
        NCOND[1] = weightZeroModel.count;
        NCOND[2] = weightOneModel.count;
        if(NCOND[0] == 0 && NCOND[1] == 0 && NCOND[2] == 0) {
            bv.MessageBox("No protocol conditions available!");
        }
        bv.SetPluginStringParameter("NCon_min", NCOND[0]);
        bv.SetPluginStringParameter("NCon_zero", NCOND[1]);
        bv.SetPluginStringParameter("NCon_plus", NCOND[2]);
    
        var cidx = 0;
        for (var j=0; j<3; j++) // list loop
        {
            for(var i=0; i<NCOND[j]; i++) // cond loop
            {
                if (NCOND[j] == 0) { break; }
                var CondName = ""
                if(j == 0) CondName = weightMinusOneModel.get(i).name
                if(j == 1) CondName = weightZeroModel.get(i).name
                if(j == 2) CondName = weightOneModel.get(i).name

                var CondIdentifier = cidx + " list" + j + " " +  CondName;
                var CondID = "CondID-" + cidx;
                bv.SetPluginStringParameter(CondID, CondIdentifier);
                cidx++;
            }
        }
    }

    function onClear()
    {
        bv.SetPluginStringParameter("Command", "Clear");
        
        weightMinusOneModel.clear(); weightZeroModel.clear(); weightOneModel.clear();
        vtcsModel.clear(); voisModel.clear();
    
        bv.ExecutePlugin();
    }
    function selectStandardPPI()
    {
        bv.PrintToLog("Script> Standard PPI analysis selected");
        bv.SetPluginStringParameter("Command", "Clear");
        bv.SetPluginIntParameter("whichapproach", 0);
    
        // clear existing list
        weightMinusOneModel.clear(); weightZeroModel.clear(); weightOneModel.clear();
        vtcsModel.clear(); voisModel.clear();
    
        // dis/enable GUI options
        weightMinLbl.enabled = true; weightZeroLbl.enabled = true; weightPlusLbl.enabled = true
        weightMinusOneListView.enabled = true; weightZeroListView.enabled = true; weightOneListView.enabled = true;
        prtContainsBaselineOp.enabled = false;
    
        bv.ExecutePlugin();
    }
    function selectGenPPI()
    {
        bv.PrintToLog("Script> Generalized PPI analysis selected");
        bv.SetPluginStringParameter("Command", "Clear");
        bv.SetPluginIntParameter("whichapproach", 1);
    
        // clear existing list
        weightMinusOneModel.clear(); weightZeroModel.clear(); weightOneModel.clear();
        vtcsModel.clear(); voisModel.clear();
    
        // dis/enable GUI options
        weightMinLbl.enabled = false; weightZeroLbl.enabled = false; weightPlusLbl.enabled = false
        weightMinusOneListView.enabled = false; weightZeroListView.enabled = false; weightOneListView.enabled = false;
        prtContainsBaselineOp.enabled = true;
        
        bv.ExecutePlugin();
    }
}
