import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: page1
    width: 400
    height: 250
    
    property alias voiFileNameTF: voiFileNameTF
    property alias subjectSpecificVOIsOp: subjectSpecificVOIsOp
    property alias voisListView: voisListView
    property alias vtcsListView: vtcsListView

    ColumnLayout {
        id: mainLayout
        anchors.fill: parent
        anchors.margins: 0
        spacing: 6

        RowLayout {
            Layout.fillHeight: true
            spacing: 16

            // VOIs
            ColumnLayout {
                Layout.fillHeight: true
                Layout.maximumWidth: 280

                Label {
                    text: "VOIs:"
                }

                GroupBox {
                    id: voisGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: voisListView
                        anchors.fill: parent
                        //Layout.fillWidth: true
                        //Layout.fillHeight: true
                        //width: 100; height: 180 //Layout.fillWidth: true // width: 100; height: 100 //anchors.fill: parent
                        spacing: 6
                        model: voisModel
                        delegate: Label { // contactDelegate
                            text: idx + ": " + name
                            MouseArea {
                                id: voisMouseArea
                                anchors.fill: parent
                                onClicked: {
                                    console.log("Clicked item: " + (index+1))
                                    voisListView.currentIndex = index
                                }
                            }
                        }
                        highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) voisGrpBx.background.color = root.extra_bg_col
                }

                //Item {
                //    Layout.fillHeight: true
                //}
                
                CheckBox {
                    id: subjectSpecificVOIsOp
                    text: "Use subject-specific VOIs"
                    ToolTip.text: "Subject-specific VOIs need to be defined as '[sub-ID]_[VOI]'"
                    ToolTip.visible: hovered
                    onClicked: root.onSwitchVOIOption()
                }
                RowLayout {
                    id: voiFileLayout

                    Label {
                        text: "VOI file:"
                    }
                    TextField {
                        id: voiFileNameTF
                        placeholderText: "file.voi"
                        Layout.fillWidth: true
                        readOnly: true
                    }
                    Button {
                        implicitWidth: 25
                        text: "..."
                        onClicked: root.getNewVOIFile()
                    }
                }
            }

            //Rectangle {
            //    Layout.fillHeight: true
            //    width: 2
            //    color: Qt.rgba(0.5, 0.5, 0.5, 1.0)
            //}

            // VTCs
            ColumnLayout {
                Layout.fillHeight: true

                Label {
                    text: "VTCs:"
                }

                GroupBox {
                    id: vtcsGrpBx
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    
                    ListView {
                        id: vtcsListView
                        anchors.fill: parent
                        //Layout.fillWidth: true // width: 100; height: 100 //anchors.fill: parent
                        //Layout.fillHeight: true
                        spacing: 6
                        model: vtcsModel
                        delegate: ColumnLayout { spacing: 0
                            Label { text: idx + ": <b>file:</b> " + name }
                            Label { text: idx + ": <b>path:</b> " + path }
                            MouseArea {
                                id: vtcsMouseArea
                                anchors.fill: parent
                                onClicked: {
                                    console.log("Clicked item: " + (index+1))
                                    vtcsListView.currentIndex = index
                                }
                            }
                        }
                        highlight: Rectangle { color: root.extra_hl_col; radius: 1; clip: true }
                        focus: true
                        clip: true
                    }
                    Component.onCompleted: if(bv.BVTheme) vtcsGrpBx.background.color = root.extra_bg_col
                }

                RowLayout {
                    id: vtcLayout
                    Layout.fillWidth: true //anchors.fill: parent

                    Button {
                        id: loadVTCsFromMDMFile
                        text: " Load From MDM... "
                        onClicked: root.onLoadVTCsFromMDM()
                    }
                    Item {  // spacer item
                        Layout.fillWidth: true
                    }
                    Button {
                        id: clearVTCList
                        // implicitWidth: 50
                        text: "Clear"
                        onClicked: vtcsModel.clear()
                    }
                    Button {
                        id: addSingleVTC
                        // implicitWidth: 25
                        text: "+"
                        onClicked: root.onAddVTCFile()
                    }
                }
            }
        }
    }
}
