import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    id: root
    width: 400
    height: 250

    ColumnLayout {
        anchors.fill: parent
        anchors.margins: 12

        Item { Layout.fillHeight: true } // vertical spacer item

        GroupBox {
            title: "Intensity range"
            Layout.fillWidth: true

            RowLayout {
                Item { Layout.fillWidth: true } // horizontal spacer item
                Label {
                    Layout.margins: 15
                    text: "From:"
                }
                SpinBox {
                    id: rangeFrom
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 0
                    editable: true
                }
                Item { width: 20 }
                Label {
                    text: "To:"
                }
                SpinBox {
                    id: rangeTo
                    Layout.maximumWidth: 60
                    from: 0
                    to: 255
                    value: 255
                    editable: true
                }
                Item { Layout.fillWidth: true } // horizontal spacer item
            }
        }

        Item { Layout.fillHeight: true } // vertical spacer item

        RowLayout {

            Item { Layout.fillWidth: true } // horizontal spacer item

            TextField {
                id: nvoxelsText
                placeholderText: "# voxels"
                Layout.fillWidth: true
            }
            Button {
                id: btnRun
                text: "Count Voxels"
                onClicked: { console.log("counting voxels..."); root.countVoxels() }
            }

            Item { Layout.fillWidth: true } // horizontal spacer item
        }

        Item { Layout.fillHeight: true } // vertical spacer item

        RowLayout {

            Item { Layout.fillWidth: true } // horizontal spacer item

            Button {
                id: btnRun2
                text: "Quit"
                onClicked: BrainVoyager.CloseDialog()
            }
        }
    }

    Component.onCompleted: {
        console.log("GUI script ready")
        BrainVoyager.PrintToLog("Hello from QML/JS GUI Script")
    }

    function countVoxels() {
        BrainVoyager.PrintToLog("Counting voxels in specified intensity range...")
        var docVMR = BrainVoyager.ActiveDocument
        if(docVMR == null)
            return
        var n_voxels = docVMR.DimX * docVMR.DimY * docVMR.DimZ
        BrainVoyager.PrintToLog("DimX: " + docVMR.DimX + " DimY: " + docVMR.DimY + " DimZ: " + docVMR.DimZ)
        BrainVoyager.PrintToLog("# of all voxels: " + n_voxels)
        BrainVoyager.PrintToLog("Counting voxels in range...")
        var from_val = rangeFrom.value
        var to_val = rangeTo.value
        var counter = 0, val = 0
        for(var z=0; z<docVMR.DimZ; z++) {
            for(var y=0; y<docVMR.DimY; y++) {
                for(var x=0; x<docVMR.DimX; x++) {
                    val = docVMR.GetVoxelIntensity(x, y, z)
                    if((val >= from_val) && (val <= to_val))
                        counter += 1
                }
            }
        }
        BrainVoyager.PrintToLog("# of voxels in range (" + from_val + "-" + to_val + "): " + counter);
        nvoxelsText.text = Number(counter)
    }
}
